@@ -3,7 +3,7 @@
 # Apache::Session::Lock::Semaphore
 # IPC Semaphore locking for Apache::Session
 # Copyright(c) 1998, 1999, 2000 Jeffrey William Baker (jwbaker@acm.org)
-# Distribute under the Artistic License
+# Distribute under the Perl License
 #
 ############################################################################
 
@@ -11,13 +11,13 @@ package Apache::Session::Lock::Semaphore;
 
 use strict;
 use Config;
-use IPC::SysV qw(IPC_CREAT S_IRWXU SEM_UNDO);
+use IPC::SysV qw(IPC_PRIVATE IPC_CREAT S_IRWXU SEM_UNDO);
 use IPC::Semaphore;
 use vars qw($VERSION);
 
-$VERSION = '1.01';
+$VERSION = '1.02';
 
-sub BEGIN {
+BEGIN {
 
     if ($Config{'osname'} eq 'linux') {
         #More semaphores on Linux means less lock contention
@@ -31,13 +31,23 @@ sub BEGIN {
 }
 
 sub new {
+    return unless $Config{d_semget};
+    return
+        if $^O eq 'cygwin' && (!exists $ENV{'CYGWIN'} || $ENV{'CYGWIN'} !~ /server/i);
+    #Modified by Alexandr Ciornii, 2007-03-12
+
     my $class   = shift;
     my $session = shift;
     
     my $nsems = $session->{args}->{NSems} ||
         $Apache::Session::Lock::Semaphore::nsems;
     
-    my $sem_key = $session->{args}->{SemaphoreKey} ||
+#    die "You shouldn't set session argument SemaphoreKey to undef"
+#     if exists($session->{args}->{SemaphoreKey}) && 
+#        !defined ($session->{args}->{SemaphoreKey});
+
+    my $sem_key = #exists ($session->{args}->{SemaphoreKey})?
+        $session->{args}->{SemaphoreKey} || 
         $Apache::Session::Lock::Semaphore::sem_key;
 
     return bless {read => 0, write => 0, sem => undef, nsems => $nsems, 
@@ -52,14 +62,15 @@ sub acquire_read_lock  {
     return if $self->{write};
 
     if (!$self->{sem}) {    
-        $self->{sem} = new IPC::Semaphore($self->{sem_key}, $self->{nsems},
-            IPC_CREAT | S_IRWXU) || die $!;
+        $self->{sem} = new IPC::Semaphore(
+            defined($self->{sem_key})?$self->{sem_key}:IPC_PRIVATE, $self->{nsems},
+            IPC_CREAT | S_IRWXU) || die "Cannot create semaphore with key $self->{sem_key}: $!";
     }
     
     if (!defined $self->{read_sem}) {
         #The number of semaphores (2^2-2^4, typically) is much less than
-        #the potential number of session ids (2^128, typically), we need 
-        #to hash the session id to choose a sempahore.  This hash routine
+        #the potential number of session ids (2^128, typically), we need
+        #to hash the session id to choose a semaphore.  This hash routine
         #was stolen from Kernighan's The Practice of Programming.
 
         my $read_sem = 0;
@@ -71,8 +82,8 @@ sub acquire_read_lock  {
         $self->{read_sem} = $read_sem;
     }    
     
-    #The sempahore block is divided into two halves.  The lower half
-    #holds the read sempahores, and the upper half holds the write
+    #The semaphore block is divided into two halves.  The lower half
+    #holds the read semaphores, and the upper half holds the write
     #semaphores.  Thus we can do atomic upgrade of a read lock to a
     #write lock.
     
@@ -89,14 +100,15 @@ sub acquire_write_lock {
     return if($self->{write});
 
     if (!$self->{sem}) {
-        $self->{sem} = new IPC::Semaphore($self->{sem_key}, $self->{nsems}, 
-            IPC_CREAT | S_IRWXU) || die $!;
+        $self->{sem} = new IPC::Semaphore(
+            defined($self->{sem_key})?$self->{sem_key}:IPC_PRIVATE, $self->{nsems},
+            IPC_CREAT | S_IRWXU) || die "Cannot create semaphore with key $self->{sem_key}: $!";
     }
     
     if (!defined $self->{read_sem}) {
         #The number of semaphores (2^2-2^4, typically) is much less than
         #the potential number of session ids (2^128, typically), we need 
-        #to hash the session id to choose a sempahore.  This hash routine
+        #to hash the session id to choose a semaphore.  This hash routine
         #was stolen from Kernighan's The Practice of Programming.
 
         my $read_sem = 0;
@@ -170,14 +182,15 @@ sub hash {
 
 =head1 NAME
 
-Apache::Session::Lock::Semaphore - Provides mutual exclusion through sempahores
+Apache::Session::Lock::Semaphore - Provides mutual exclusion through semaphores
 
 =head1 SYNOPSIS
 
  use Apache::Session::Lock::Semaphore;
- 
+
  my $locker = new Apache::Session::Lock::Semaphore;
- 
+ die "no semaphores" unless $locker;
+
  $locker->acquire_read_lock($ref);
  $locker->acquire_write_lock($ref);
  $locker->release_read_lock($ref);
@@ -186,27 +199,27 @@ Apache::Session::Lock::Semaphore - Provides mutual exclusion through sempahores
 
 =head1 DESCRIPTION
 
-Apache::Session::Lock::Sempahore fulfills the locking interface of 
+Apache::Session::Lock::semaphore fulfills the locking interface of 
 Apache::Session.  Mutual exclusion is achieved through system semaphores and
 the IPC::Semaphore module.
 
 =head1 CONFIGURATION
 
 The module must know how many semaphores to use, and what semaphore key to
-use.  The number of semaphores has an impact on performance.  More semaphores
-meansless lock contention.  You should use the maximum number of sempahores
-that your platform will allow.  On stock NetBSD, OpenBSD, and Solaris systems,
-this is probably 16.  On Linux 2.2, this is 32.  This module tries to guess
+use. The number of semaphores has an impact on performance.  More semaphores
+means less lock contention. You should use the maximum number of semaphores
+that your platform will allow. On stock NetBSD, OpenBSD, and Solaris systems,
+this is probably 16. On Linux 2.2, this is 32. This module tries to guess
 the number based on your operating system, but it is safer to configure it
 yourself.
 
 To set the number of semaphores, you need to pass an argument in the usual
-Apache::Session style.  The name of the argument is NSems, and the value is
-an integer power of 2.  For example:
+Apache::Session style. The name of the argument is NSems, and the value is
+an integer power of 2. For example:
 
  tie %s, 'Apache::Session::Blah', $id, {NSems => 16};
 
-You may also need to configure the semaphore key that this package uses.  By
+You may also need to configure the semaphore key that this package uses. By
 default, it uses key 31818.  You can change this using the argument
 SemaphoreKey:
 
@@ -220,8 +233,8 @@ package.
 If you get an invalid argument message, that usually means that the system
 is unhappy with the number of semaphores that you requested.  Try decreasing
 the number of semaphores.  The semaphore blocks that this package creates
-are persistent until the system is rebooted, so if you request 8 sempahores
-one time and 16 sempahores the next, it won't work.  Use the system
+are persistent until the system is rebooted, so if you request 8 semaphores
+one time and 16 semaphores the next, it won't work.  Use the system
 commands ipcs and ipcrm to inspect and remove unwanted semphore blocks.
 
 =head2 Cygwin
@@ -231,6 +244,17 @@ IPC on Cygwin requires running cygserver. Without it, program will exit with
 
 Read /usr/share/doc/Cygwin/cygserver.README for more information.
 
+=head2 Darwin/MacOS X
+
+Darwin and MacOS X may not have semaphores, see
+L<http://sysnet.ucsd.edu/~bellardo/darwin/sysvsem.html>
+
+=head2 *BSD
+
+Error "No space left on device" means that maximum number of semaphores is reached.
+See L<http://www.postgresql.org/docs/7.3/static/kernel-resources.html> for
+more information.
+
 =head1 AUTHOR
 
 This module was written by Jeffrey William Baker <jwbaker@acm.org>.
